package net.sergeych.crypto2

import kotlinx.serialization.Serializable
import kotlinx.serialization.Transient

@Serializable
class UniversalRing(
    private val keys: Collection<UniversalKey>
): Collection<UniversalKey> by keys {
    constructor(vararg keys: UniversalKey) : this(keys.toSet())
    constructor(vararg keys: DecryptingKey) : this(keys.map { UniversalKey.from(it) }.toSet())
    @Transient
    val keySet = if( keys is Set<UniversalKey> ) keys else keys.toSet()

    private val byTag by lazy { keySet.associateBy { it.tag } }

    operator fun get(keyTag: KeyTag): UniversalKey? = byTag[keyTag]

    override operator fun contains(element: UniversalKey): Boolean = byTag.containsKey(element.tag)

    operator fun plus(key: UniversalKey): UniversalRing =
        if( key in this ) this else UniversalRing(keySet + key )

    operator fun minus(key: UniversalKey): UniversalRing =
        if( key in this ) UniversalRing(keySet.filter { it.tag != key.tag }) else this

    operator fun minus(keyTag: KeyTag): UniversalRing =
        if( keyTag in byTag ) UniversalRing(keySet.filter { it.tag != keyTag }) else this

    override fun equals(other: Any?): Boolean {
        println("compare1\r\n")
        if (this === other) return true
        println("compare2\r\n")
        if (other !is UniversalRing) return false

        println("compare2 ${size == other.size}: $size : ${other.size} | ${keySet == other.keySet}\r\n")
        return size == other.size && keySet.containsAll(other.keySet)
    }

    override fun toString(): String {
        return "Kr[${keys.joinToString(",")}]"
    }

    override fun hashCode(): Int {
        return keySet.hashCode()
    }

}
